@@ -21,16 +21,18 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
-
+from abc import abstractmethod
+import sys
 import http.cookies
+import inspect
 import time
 from typing import Any, Dict, List, Tuple, Union, Callable
 from simple_http_server.logger import get_logger
 
 name = "simple_http_server"
-version = "0.5.8"
+version = "0.7.0"
 
-__logger = get_logger("simple_http_server.__init__")
+_logger = get_logger("simple_http_server.__init__")
 
 
 class Session:
@@ -94,6 +96,7 @@ class Request:
         self.__cookies = Cookies()
         self.query_string: str = ""  # Query String
         self.path_values: Dict[str, str] = {}
+        self.reg_groups = ()  # If controller is matched via regexp, then ,all groups are save here
         self.path: str = ""  # Path
         self.__parameters = {}  # Parameters, key-value array, merged by query string and request body if the `Content-Type` in request header is `application/x-www-form-urlencoded` or `multipart/form-data`
         self.__parameter = {}  # Parameters, key-value, if more than one parameters with the same key, only the first one will be stored.
@@ -125,6 +128,7 @@ class Request:
         else:
             return self.parameter[key]
 
+    @abstractmethod
     def get_session(self, create: bool = False) -> Session:
         # This is abstract method
         return None
@@ -234,9 +238,33 @@ class Parameters(list):
         obj.extend(default)
         return obj
 
+
 class ModelDict(dict):
     pass
 
+
+class RegGroups(tuple):
+    pass
+
+
+class RegGroup(str):
+
+    def __init__(self, group=0, **kwargs):
+        self._group = group
+
+    @property
+    def group(self) -> int:
+        return self._group
+
+    def __new__(cls, group=0, **kwargs):
+        if "_value" not in kwargs:
+            val = ""
+        else:
+            val = kwargs["_value"]
+        obj = super().__new__(cls, val)
+        return obj
+
+
 class Header(ParamStringValue):
     pass
 
@@ -379,35 +407,169 @@ class Cookie(http.cookies.Morsel):
         return self.__required
 
 
-__request_mappings = []
-__filters = []
-
-__session_facory: SessionFactory = None
+def _get_class_of_method(method_defind):
+    vals = vars(sys.modules[method_defind.__module__])
+    for attr in method_defind.__qualname__.split('.')[:-1]:
+        if attr in vals:
+            vals = vals[attr]
+    if inspect.isclass(vals):
+        return vals
 
 
-def request_map(url: str = "", method: Union[str, list] = "", controller_function: Callable = None) -> Callable:
-    def map(ctrl_fun):
-        if isinstance(method, list):
-            mths = method
+def _create_object(clz, args=[], kwargs={}):
+
+    if clz is None:
+        return None
+    elif args and kwargs:
+        return clz(*args, **kwargs)
+    elif args:
+        return clz(*args)
+    elif kwargs:
+        return clz(**kwargs)
+    else:
+        return clz()
+
+
+class ControllerFunction:
+
+    def __init__(self, url: str = "",
+                 regexp: str = "",
+                 method: str = "",
+                 ctr_obj: object = None,
+                 func: Callable = None) -> None:
+        self.__url: str = url
+        self.__regexp = regexp
+        self.__method: str = method
+        self.singletion: bool = False
+        self.ctr_obj_init_args = None
+        self.ctr_obj_init_kwargs = None
+        self.__ctr_obj: object = ctr_obj
+        self.__func: Callable = func
+        self.__clz = False
+
+    @property
+    def _is_config_ok(self):
+        try:
+            assert self.url or self.regexp, "you should set one of url and regexp"
+            assert not (self.url and self.regexp), "you can only set one of url and regexp, not both"
+            assert self.func is not None and (inspect.isfunction(self.func) or inspect.ismethod(self.func))
+            return True
+        except AssertionError as ae:
+            _logger.warn(f"[{self._url}|{self.regexp}] => {self.func} configurate error: {ae}")
+            return False
+
+    @property
+    def url(self) -> str:
+        return self.__url
+
+    @property
+    def regexp(self) -> str:
+        return self.__regexp
+
+    @property
+    def method(self) -> str:
+        return self.__method
+
+    @property
+    def clz(self) -> str:
+        if self.__clz != False:
+            return self.__clz
+        if inspect.isfunction(self.func):
+            self.__clz = _get_class_of_method(self.func)
         else:
-            mths = [method]
-        for mth in mths:
-            __logger.debug("map url %s with method[%s] to function %s. " % (url, mth, str(ctrl_fun)))
-            __request_mappings.append({
-                "url": url,
-                "method": mth,
-                "func": ctrl_fun
-            })
-        # return the original function, so you can use a decoration chain
-        return ctrl_fun
-    if controller_function:
-        map(controller_function)
+            self.__clz = None
+        return self.__clz
+
+    @property
+    def ctrl_object(self) -> object:
+        if not self.singletion:
+            obj = self._create_ctrl_obj()
+            _logger.debug(f"singleton: create a object -> {obj}")
+            return obj
+
+        if self.__ctr_obj is None:
+            self.__ctr_obj = self._create_ctrl_obj()
+            _logger.debug(f"object does not exist, create one -> {self.__ctr_obj} ")
+        else:
+            _logger.debug(f"object[{self.__ctr_obj}] exists, return. ")
+        return self.__ctr_obj
+
+    def _create_ctrl_obj(self) -> object:
+        return _create_object(self.clz, self.ctr_obj_init_args, self.ctr_obj_init_kwargs)
+
+    @ctrl_object.setter
+    def ctrl_object(self, val) -> None:
+        self.__ctr_obj = val
+
+    @property
+    def func(self) -> Callable:
+        return self.__func
+
+
+_request_mappings: List[ControllerFunction] = []
+_request_clz_mapping: Dict[Any, Tuple[str, Union[str, list]]] = {}
+
+_filters = []
+
+_ctrls = {}
+_ctrl_singletons = {}
+
+_session_facory: SessionFactory = None
+
+
+def controller(*anno_args, singleton: bool = True, args: List[Any] = [], kwargs: Dict[str, Any] = {}):
+    def map(ctr_obj_class):
+        _logger.debug(f"map controller[{ctr_obj_class}]({singleton}) with args: {args}, and kwargs: {kwargs})")
+        _ctrls[ctr_obj_class] = (singleton, args, kwargs)
+        return ctr_obj_class
+    if len(anno_args) == 1 and inspect.isclass(anno_args[0]):
+        return map(anno_args[0])
     return map
 
 
-def filter_map(pattern: str = "", filter_function: Callable = None) -> Callable:
+def request_map(*anno_args, url: str = "", regexp: str = "", method: Union[str, list, tuple] = "") -> Callable:
+    _url = url
+    len_args = len(anno_args)
+    assert len_args <= 1
+
+    arg_ctrl = None
+    if len_args == 1:
+        if isinstance(anno_args[0], str):
+            assert not url and not regexp
+            _url = anno_args[0]
+        elif callable(anno_args[0]):
+            arg_ctrl = anno_args[0]
+
+    def map(ctrl):
+        if isinstance(method, list):
+            mths = method
+        elif isinstance(method, tuple):
+            mths = list(method)
+        else:
+            mths = [m.strip() for m in method.split(',')]
+
+        if inspect.isclass(ctrl):
+            _request_clz_mapping[ctrl] = (_url, mths)
+            return ctrl
+
+        for mth in mths:
+            _logger.debug(f"map url {_url} with method[{mth}] to function {ctrl}. ")
+            _request_mappings.append(ControllerFunction(url=_url, regexp=regexp, method=mth, func=ctrl))
+        # return the original function, so you can use a decoration chain
+        return ctrl
+
+    if arg_ctrl:
+        return map(arg_ctrl)
+    else:
+        return map
+
+
+route = request_map
+
+
+def filter_map(regexp: str = "", filter_function: Callable = None) -> Callable:
     def map(filter_fun):
-        __filters.append({"url_pattern": pattern, "func": filter_fun})
+        _filters.append({"url_pattern": regexp, "func": filter_fun})
         return filter_fun
     if filter_function:
         map(filter_function)
@@ -415,17 +577,65 @@ def filter_map(pattern: str = "", filter_function: Callable = None) -> Callable:
 
 
 def set_session_factory(session_factory: SessionFactory):
-    global __session_facory
-    __session_facory = session_factory
+    global _session_facory
+    _session_facory = session_factory
+
+
+def _get_singletion(clz, args, kwargs):
+    if clz not in _ctrl_singletons:
+        _ctrl_singletons[clz] = _create_object(clz, args, kwargs)
+    return _ctrl_singletons[clz]
 
 
 def _get_request_mappings():
-    return __request_mappings
+    mappings: List[ControllerFunction] = []
+
+    for ctr_fun in _request_mappings:
+        clz = ctr_fun.clz
+        if clz is not None and clz in _request_clz_mapping:
+            clz_url, methods = _request_clz_mapping[clz]
+
+            if ctr_fun.regexp:
+                full_url = ctr_fun.url
+            else:
+                fun_url = ctr_fun.url
+                if fun_url:
+                    if not clz_url.endswith("/"):
+                        clz_url = clz_url + "/"
+                    if fun_url.startswith("/"):
+                        fun_url = fun_url[1:]
+                    full_url = f"{clz_url}{fun_url}"
+                else:
+                    full_url = clz_url
+
+            if not ctr_fun.method and methods:
+                for mth in methods:
+                    _logger.debug(f"map url {full_url} included [{clz_url}] with method[{mth}] to function {ctr_fun.func}. ")
+                    mappings.append(ControllerFunction(url=full_url, regexp=ctr_fun.regexp, method=mth, func=ctr_fun.func))
+            else:
+                _logger.debug(f"map url {full_url} included [{clz_url}] with method[{ctr_fun.method}] to function {ctr_fun.func}. ")
+                mappings.append(ControllerFunction(url=full_url, regexp=ctr_fun.regexp, method=ctr_fun.method, func=ctr_fun.func))
+        else:
+            mappings.append(ctr_fun)
+
+    for ctr_fun in mappings:
+        if not ctr_fun._is_config_ok:
+            continue
+        clz = ctr_fun.clz
+        if clz is not None and clz in _ctrls:
+            singleton, args, kwargs = _ctrls[clz]
+            ctr_fun.singletion = singleton
+            ctr_fun.ctr_obj_init_args = args
+            ctr_fun.ctr_obj_init_kwargs = kwargs
+            if singleton:
+                ctr_fun.ctrl_object = _get_singletion(clz, args, kwargs)
+
+    return mappings
 
 
 def _get_filters():
-    return __filters
+    return _filters
 
 
 def _get_session_factory() -> SessionFactory:
-    return __session_facory
+    return _session_facory
