@@ -37,25 +37,11 @@ from collections import OrderedDict
 from socketserver import ThreadingMixIn
 from urllib.parse import unquote
 from urllib.parse import quote
-from typing import Dict, Tuple
+from typing import Callable, Dict, List, Tuple
 
-from simple_http_server import ModelDict, _get_session_factory
-from simple_http_server import HttpError
-from simple_http_server import StaticFile
-from simple_http_server import Headers
-from simple_http_server import Redirect
-from simple_http_server import Response
-from simple_http_server import Cookies
-from simple_http_server import Cookie
-from simple_http_server import JSONBody
-from simple_http_server import Header
-from simple_http_server import Parameters
-from simple_http_server import PathValue
-from simple_http_server import Parameter
-from simple_http_server import MultipartFile
-from simple_http_server import Request
-from simple_http_server import Session
-from simple_http_server import version
+from simple_http_server import ModelDict, RegGroup, RegGroups, HttpError, StaticFile, \
+    Headers, Redirect, Response, Cookies, Cookie, JSONBody, Header, Parameters, PathValue, \
+    Parameter, MultipartFile, Request, Session, ControllerFunction, version, _get_session_factory
 from simple_http_server._http_session_local_impl import SESSION_COOKIE_NAME
 
 from simple_http_server.logger import get_logger
@@ -124,10 +110,10 @@ class ResponseWrapper(Response):
 class FilterContex:
     """Context of a filter"""
 
-    def __init__(self, req, res, controller, filters=None):
+    def __init__(self, req, res, controller: ControllerFunction, filters=None):
         self.__request = req
         self.__response = res
-        self.__controller = controller
+        self.__controller: ControllerFunction = controller
         self.__filters = filters if filters is not None else []
 
     @property
@@ -138,17 +124,22 @@ class FilterContex:
     def response(self) -> Response:
         return self.__response
 
+    def _run_ctroller(self):
+        args = self.__prepare_args()
+        kwargs = self.__prepare_kwargs()
+
+        if kwargs is None:
+            ctr_res = self.__controller.func(*args)
+        else:
+            ctr_res = self.__controller.func(*args, **kwargs)
+        return ctr_res
+
     def do_chain(self):
         if self.response.is_sent:
             return
         if len(self.__filters) == 0:
-            args = self.__prepare_args()
-            kwargs = self.__prepare_kwargs()
 
-            if kwargs is None:
-                ctr_res = self.__controller(*args)
-            else:
-                ctr_res = self.__controller(*args, **kwargs)
+            ctr_res = self._run_ctroller()
 
             session = self.request.get_session()
             if session and session.is_valid:
@@ -218,13 +209,20 @@ class FilterContex:
         return status_code, headers, cks, body
 
     def __prepare_args(self):
-        args = _get_args_(self.__controller)
+        args = _get_args_(self.__controller.func)
         arg_vals = []
+        if len(args) > 0:
+            ctr_obj = self.__controller.ctrl_object
+            if ctr_obj is not None:
+                arg_vals.append(self.__controller.ctrl_object)
+                args = args[1:]
         for arg, arg_type_anno in args:
-            if arg not in self.request.parameter.keys() and arg_type_anno not in (Request, Session, Response, Headers, cookies.BaseCookie, cookies.SimpleCookie, Cookies, PathValue, JSONBody, ModelDict):
+            if arg not in self.request.parameter.keys() \
+                    and arg_type_anno not in (Request, Session, Response, RegGroups, RegGroup, Headers, cookies.BaseCookie, cookies.SimpleCookie, Cookies, PathValue, JSONBody, ModelDict):
                 raise HttpError(400, f"Parameter[{arg}] is required! ")
             param = self.__get_params_(arg, arg_type_anno)
             arg_vals.append(param)
+
         return arg_vals
 
     def __get_params_(self, arg, arg_type, val=None, type_check=True):
@@ -241,9 +239,11 @@ class FilterContex:
             param = self.response
         elif arg_type == Headers:
             param = Headers(self.request.headers)
+        elif arg_type == RegGroups:
+            param = RegGroups(self.request.reg_groups)
         elif arg_type == Header:
             param = self.__build_header(arg, **kws)
-        elif issubclass(arg_type, cookies.BaseCookie):
+        elif inspect.isclass(arg_type) and issubclass(arg_type, cookies.BaseCookie):
             param = self.request.cookies
         elif arg_type == Cookie:
             param = self.__build_cookie(arg, **kws)
@@ -255,6 +255,8 @@ class FilterContex:
             param = self.__build_path_value(arg, **kws)
         elif arg_type == Parameters:
             param = self.__build_params(arg, **kws)
+        elif arg_type == RegGroup:
+            param = self.__build_reg_group(**kws)
         elif arg_type == JSONBody:
             param = self.__build_json_body()
         elif arg_type == str:
@@ -265,11 +267,11 @@ class FilterContex:
             param = self.__build_int(arg, **kws)
         elif arg_type == float:
             param = self.__build_float(arg, **kws)
-        elif arg_type == list:
-            param = self.__build_list(arg, **kws)
+        elif arg_type in (list, List, List[str], List[Parameter], List[int], List[float], List[bool], List[dict], List[Dict]):
+            param = self.__build_list(arg, target_type=arg_type, **kws)
         elif arg_type == ModelDict:
             param = self.__build_model_dict()
-        elif arg_type == dict:
+        elif arg_type in (dict, Dict):
             param = self.__build_dict(arg, **kws)
         elif type_check:
             raise HttpError(400, f"Parameter[{arg}] with Type {arg_type} is not supported yet.")
@@ -277,6 +279,11 @@ class FilterContex:
             param = val
         return param
 
+    def __build_reg_group(self, val: RegGroup = RegGroup(group=0)):
+        if val.group >= len(self.request.reg_groups):
+            raise HttpError(400, f"RegGroup required an element at {val.group}, but the reg length is only {len(self.request.reg_groups)}")
+        return RegGroup(group=val.group, _value=self.request.reg_groups[val.group])
+
     def __build_model_dict(self):
         mdict = ModelDict()
         for k, v in self.request.parameters.items():
@@ -287,7 +294,7 @@ class FilterContex:
         return mdict
 
     def __prepare_kwargs(self):
-        kwargs = _get_kwargs_(self.__controller)
+        kwargs = _get_kwargs_(self.__controller.func)
         if kwargs is None:
             return None
         kwarg_vals = {}
@@ -334,15 +341,37 @@ class FilterContex:
             try:
                 return json.loads(self.request.parameter[key])
             except:
-                raise HttpError(400, f"Parameter[{key}] should be a JSON type string.")
+                raise HttpError(400, f"Parameter[{key}] should be a JSON string.")
         else:
             return val
 
-    def __build_list(self, key, val=[]):
+    def __build_list(self, key, target_type=list, val=[]):
         if key in self.request.parameters.keys():
-            return self.request.parameters[key]
+            ori_list = self.request.parameters[key]
         else:
-            return val
+            ori_list = val
+
+        if target_type == List[int]:
+            try:
+                return [int(p) for p in ori_list]
+            except:
+                raise HttpError(400, f"One of the parameter[{key}] is not int. ")
+        elif target_type == List[float]:
+            try:
+                return [float(p) for p in ori_list]
+            except:
+                raise HttpError(400, f"One of the parameter[{key}] is not float. ")
+        elif target_type == List[bool]:
+            return [p.lower() not in ("0", "false", "") for p in ori_list]
+        elif target_type in (List[dict], List[Dict]):
+            try:
+                return [json.loads(p) for p in ori_list]
+            except:
+                raise HttpError(400, f"One of the parameter[{key}] is not JSON string. ")
+        elif target_type == List[Parameter]:
+            return [Parameter(name=key, default=p, required=False) for p in ori_list]
+        else:
+            return ori_list
 
     def __build_float(self, key, val=None):
         if key in self.request.parameter.keys():
@@ -416,7 +445,7 @@ class FilterContex:
 
 def _get_args_(func):
     argspec = inspect.getfullargspec(func)
-    # ignore `self`
+    # ignore first argument like `self` or `clz` in object methods or class methods
     start = 1 if inspect.ismethod(func) else 0
     if argspec.defaults is None:
         args = argspec.args[start:]
@@ -469,7 +498,7 @@ class _SimpleDispatcherHttpRequestHandler(http.server.BaseHTTPRequestHandler):
         req = self.__prepare_request(mth)
         path = req._path
 
-        ctrl, req.path_values = self.server.get_url_controller(path, mth)
+        ctrl, req.path_values, req.reg_groups = self.server.get_url_controller(path, mth)
 
         res = ResponseWrapper(self)
         if ctrl is None:
@@ -491,7 +520,7 @@ class _SimpleDispatcherHttpRequestHandler(http.server.BaseHTTPRequestHandler):
                 res.body = {"error":  str(e)}
                 res.send_response()
 
-    def __prepare_request(self, method):
+    def __prepare_request(self, method) -> RequestWrapper:
         path = self.__get_path(self.path)
         req = RequestWrapper()
         req.path = "/" + path
@@ -627,10 +656,10 @@ class _SimpleDispatcherHttpRequestHandler(http.server.BaseHTTPRequestHandler):
         else:
             params[key].append(val)
 
-    def _send_response(self, response):        
+    def _send_response(self, response):
         headers = response["headers"]
         cks = response["cookies"]
-        
+
         status_code, content_type, body = self.__prepare_res_body(response)
 
         if "Content-Type" not in headers.keys() and "content-type" not in headers.keys():
@@ -654,11 +683,11 @@ class _SimpleDispatcherHttpRequestHandler(http.server.BaseHTTPRequestHandler):
             self.send_header("Content-Length", 0)
             self.end_headers()
         elif isinstance(body, str):
-                data = body.encode("utf-8")
-                self.send_header("Content-Length", len(data))
-                self.end_headers()
-                self.wfile.write(data)
-            
+            data = body.encode("utf-8")
+            self.send_header("Content-Length", len(data))
+            self.end_headers()
+            self.wfile.write(data)
+
         elif isinstance(body, bytes):
             self.send_header("Content-Length", len(body))
             self.end_headers()
@@ -749,11 +778,13 @@ class _HttpServerWrapper(http.server.HTTPServer):
 
     def __init__(self, addr, res_conf={}):
         super(_HttpServerWrapper, self).__init__(addr, _SimpleDispatcherHttpRequestHandler)
-        self.method_url_mapping = {"_": {}}
-        self.path_val_url_mapping = {"_": {}}
+        self.method_url_mapping: Dict[str, Dict[str, ControllerFunction]] = {"_": {}}
+        self.path_val_url_mapping: Dict[str, Dict[str, ControllerFunction]] = {"_": OrderedDict()}
+        self.method_regexp_mapping: Dict[str, Dict[str, ControllerFunction]] = {"_": OrderedDict()}
         for mth in _HttpServerWrapper.HTTP_METHODS:
             self.method_url_mapping[mth] = {}
-            self.path_val_url_mapping[mth] = {}
+            self.path_val_url_mapping[mth] = OrderedDict()
+            self.method_regexp_mapping[mth] = OrderedDict()
 
         self.filter_mapping = OrderedDict()
         self._res_conf = []
@@ -793,7 +824,7 @@ class _HttpServerWrapper(http.server.HTTPServer):
         self._res_conf.sort(key=lambda it: -len(it[0]))
 
     def __get_path_reg_pattern(self, url):
-        _url = url
+        _url: str = url
         path_names = re.findall("(?u)\\{\\w+\\}", _url)
         if len(path_names) == 0:
             # normal url
@@ -808,18 +839,23 @@ class _HttpServerWrapper(http.server.HTTPServer):
             quoted_names.append(quote(name))
         return _url, quoted_names
 
-    def map_url(self, url, func, method=""):
-        assert url is not None
-        assert func is not None and (inspect.isfunction(func) or inspect.ismethod(func))
+    def map_url(self, ctrl: ControllerFunction):
+        url = ctrl.url
+        regexp = ctrl.regexp
+        method = ctrl.method
+        _logger.debug(f"map url {url}|{regexp} with method[{method}] to function {ctrl.func}. ")
         assert method is None or method == "" or method.upper() in _HttpServerWrapper.HTTP_METHODS
         _method = method.upper() if method is not None and method != "" else "_"
-        _url = _remove_url_first_slash(url)
-
-        path_pattern, path_names = self.__get_path_reg_pattern(_url)
-        if path_pattern is None:
-            self.method_url_mapping[_method][_url] = func
+        if regexp:
+            self.method_regexp_mapping[_method][regexp] = ctrl
         else:
-            self.path_val_url_mapping[_method][path_pattern] = (func, path_names)
+            _url = _remove_url_first_slash(url)
+
+            path_pattern, path_names = self.__get_path_reg_pattern(_url)
+            if path_pattern is None:
+                self.method_url_mapping[_method][_url] = ctrl
+            else:
+                self.path_val_url_mapping[_method][path_pattern] = (ctrl, path_names)
 
     def _res_(self, path, res_pre, res_dir):
         fpath = os.path.join(res_dir, path.replace(res_pre, ""))
@@ -851,36 +887,52 @@ class _HttpServerWrapper(http.server.HTTPServer):
 
         return StaticFile(fpath, content_type)
 
-    def get_url_controller(self, path="", method=""):
+    def get_url_controller(self, path="", method="") -> Tuple[Callable, Dict, List]:
+        # explicitly url matching
         if path in self.method_url_mapping[method]:
-            return self.method_url_mapping[method][path], {}
+            return self.method_url_mapping[method][path], {}, ()
         elif path in self.method_url_mapping["_"]:
-            return self.method_url_mapping["_"][path], {}
-        else:
-            fun_and_val = self.__try_get_from_path_val(path, method)
-            if fun_and_val is None:
-                fun_and_val = self.__try_get_from_path_val(path, "_")
-            if fun_and_val is not None:
-                return fun_and_val
-            else:
-                for k, v in self.res_conf:
-                    if path.startswith(k):
-                        def static_fun():
-                            return self._res_(path, k, v)
-                        return static_fun, {}
-                return None, {}
+            return self.method_url_mapping["_"][path], {}, ()
+
+        # url with path value matching
+        fun_and_val = self.__try_get_from_path_val(path, method)
+        if fun_and_val is None:
+            fun_and_val = self.__try_get_from_path_val(path, "_")
+        if fun_and_val is not None:
+            return fun_and_val[0], fun_and_val[1], ()
+
+        # regexp
+        func_and_groups = self.__try_get_from_regexp(path, method)
+        if func_and_groups is None:
+            func_and_groups = self.__try_get_from_regexp(path, "_")
+        if func_and_groups is not None:
+            return func_and_groups[0], {}, func_and_groups[1]
+        # static files
+        for k, v in self.res_conf:
+            if path.startswith(k):
+                def static_fun():
+                    return self._res_(path, k, v)
+                return static_fun, {}, ()
+        return None, {}, ()
+
+    def __try_get_from_regexp(self, path, method):
+        for regex, ctrl in self.method_regexp_mapping[method].items():
+            m = re.match(regex, path)
+            _logger.debug(f"regexp::pattern::[{regex}] => path::[{path}] match? {m is not None}")
+            if m:
+                return ctrl, tuple([unquote(v) for v in m.groups()])
+        return None
 
     def __try_get_from_path_val(self, path, method):
         for patterns, val in self.path_val_url_mapping[method].items():
             m = re.match(patterns, path)
-            is_match = m is not None
-            _logger.debug(f"pattern::[{patterns}] => path::[{path}] match? {is_match}")
-            if is_match:
+            _logger.debug(f"url with path value::pattern::[{patterns}] => path::[{path}] match? {m is not None}")
+            if m:
                 fun, path_names = val
                 path_values = {}
                 for idx in range(len(path_names)):
-                    key = unquote(str(path_names[idx]))
-                    path_values[key] = unquote(str(m.groups()[idx]))
+                    key = unquote(path_names[idx])
+                    path_values[key] = unquote(m.groups()[idx])
                 return fun, path_values
         return None
 
@@ -905,8 +957,8 @@ class SimpleDispatcherHttpServer(object):
     def map_filter(self, path_pattern, filter_fun):
         self.server.map_filter(path_pattern, filter_fun)
 
-    def map_request(self, url, fun, method=""):
-        self.server.map_url(url, fun, method)
+    def map_request(self, ctrl: ControllerFunction):
+        self.server.map_url(ctrl)
 
     def __init__(self,
                  host: Tuple[str, int] = ('', 9090),
